/* CDEMO.C, Demo of using WPLOTC32.DLL to send commands & data to WPLOT.   */
/* See header file WPLOTC32.H for comments on using the DLL functions.     */
/* CDEMO was compiled as a Win32 console program using Borland C/C++ 5.01. */
/* The program must be linked with WPLOTC32.LIB (Borland Import library)   */
/* or have a DEF file containing the following lines: */
/* IMPORTS  WPLOTC32.StartWPlot  */
/*          WPLOTC32.WPlotcmd    */
/* Programmer: William G. Hood, 9-16-95, 10-27-97 */
#include <stdio.h>
#include <conio.h>
#include <windows.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include "wplotc32.h"

#define ESC 27
#define ID   1

int sendc( int id, char *pcmd );
void SetWinSize( HWND hwind, int lines );

void main( void )
  {
  HWND hwind, hwind2;
  int i, errcode;
  char buffer[81];
  char *ErrMes[5] =
    { "Out of memory", "Invalid ID", "File WPLOT32.EXE not found",
      "Path to WPLOT32.EXE not found", "Unspecified Error" };
  double x, y, ydata[9] = { 125, 175, 350, 750, 900, 750, 350, 175, 125 };

  hwind = GetForegroundWindow();   /* Get handle to client window */
  SetWinSize( hwind, 13 );

  printf("\n"
    " CDEMO: Demo of using WPLOTC32.DLL to control WPlot.\n");
  printf("\n"
    " WPlot can be used to plot data collected, calculated, or\n"
    " processed by a user written C or Visual Basic program.\n");
  printf("\n"
    " This is an example of a console program that plots data by\n"
    " sending the data and plot commands to WPlot using functions in\n"
    " the Dynamic Link Library WPLOTC32.DLL; see the C source code\n"
    " in the file CDEMO.C.\n");

  printf("\n Press a key to send example data and plot commands to WPlot. ");
  while( kbhit() ) getch();   /* Flush keystroke buffer. */
  getch(); printf(" OK\n");   /* Wait until a key is pressed. */
  
  /* Launch WPlot using default path & keep this program in the foregound. */
  errcode = StartWPlot( ID, NULL, FALSE );
  if( errcode > 0 )
    {
    printf("\n ERROR: %s\n", ErrMes[ errcode < 5 ? errcode-1 : 4 ] );
    printf("\n Press a key to exit. ");
    while( kbhit() ) getch();   /* Flush keystroke buffer. */
    getch(); printf(" OK\n");   /* Wait until a key is pressed. */
    return;
    }

  sendc( ID, "winpos 0.5 45.0" );  /* Set Window position */
  sendc( ID, "plotsize 1" );       /* Set small plot size */
  if( sendc( ID, "" ) )            /* Check that above commands were received */
    {
    printf("\n ERROR: WPlot is not responding.\n" );
    printf("\n Press a key to exit. ");
    while( kbhit() ) getch();   /* Flush keystroke buffer. */
    getch(); printf(" OK\n");   /* Wait until a key is pressed. */
    return;
    }

  sendc( ID, "title Demo of Controlling WPlot" );
  sendc( ID, "xlabel X-Axis" );
  sendc( ID, "ylabel Y-Axis" );
  sendc( ID, "xstart 0" ); sendc( ID, "xend 10" );   sendc( ID, "xstep 1" );
  sendc( ID, "ystart 0" ); sendc( ID, "yend 1000" ); sendc( ID, "ystep 200" );
  sendc( ID, "grids" );
  sendc( ID, "read 9" );
  for( i = 0; i < 9; ++i )
    {
    sprintf( buffer, "%d %f", i+1, ydata[i] );
    sendc( ID, buffer );   /* Data points can be sent after a READ command. */
    }
  sendc( ID, "curve 1" );  /* The data set will be plotted as a smooth curve.*/
  sendc( ID, "symbols" );  /* A symbol will be plotted at each data point.   */
  sendc( ID, "plot" );     /* Plot previously read data points.              */

  printf("\n Press a key to plot data points as they are calculated. ");
  while( kbhit() ) getch();   /* Flush keystroke buffer. */
  getch(); printf(" OK\n");   /* Wait until a key is pressed. */

  /* Some special commands were added to WPlot for real-time data display: */
  /* rtdata   m     - Allocates memory for a maximum of m data points.     */
  /* rtpoint  x  y  - Plots point (x,y).                                   */
  /* rtend          - Terminates real-time plot & frees Windows resources. */
  sendc( ID, "rtdata 51" );
  for( i = 0; i < 51; ++i )
    {
    x = i / 5.0;
    y = x * x * x + 2.0 * x * x - 30.0 * x + 90.0;
    y += 20.0 * ( 2.0 * rand() - RAND_MAX ) / RAND_MAX;
    printf(" x = %7.2f,  y = %7.2f\n", x, y );
    sprintf( buffer, "rtpoint %7.2f %7.2f", x, y );
    sendc( ID, buffer );
    }
  sendc( ID, "rtend" );

  printf("\n Press a key to make a polynomial curve fit. ");
  while( kbhit() ) getch();   /* Flush keystroke buffer. */
  getch(); printf(" OK\n");   /* Wait until a key is pressed. */

  sendc( ID, "poly 3" );
  sendc( ID, "title Polynomial Curve Fit" );
  sendc( ID, "plot" );

  printf("\n Press a key to exit. ");
  while( kbhit() ) getch();   /* Flush keystroke buffer. */
  getch(); printf(" OK\n");   /* Wait until a key is pressed. */

  sendc( ID, "exit" );
  DestroyWindow( hwind );
  }

int sendc( int id, char *pcmd )
/* Sends command pointed to by pcmd to WPLOTC32.DLL using buffer #id.      */
/* The function keeps trying until the DLL is ready to accept the command. */
/* Returns 0 on success or 1 if times out after 20 seconds.                */
  {
  long tstart;

  tstart = time(NULL);
  while( 1 )
    {
    if( WPlotcmd( id, pcmd, 0 ) == 0 ) return 0;
    if( time(NULL) - tstart > 20 ) break;
    }
  printf("ERROR: send command function timed out.\n");
  return 1;
  }

void SetWinSize( HWND hwind, int lines )
/* Sets client window's position and sets size to specified number of lines */
  {
  int texth, textw;
  RECT rect;
  HDC hdc;
  TEXTMETRIC tm;

  SendMessage( hwind, WM_SETFONT,
    (WPARAM) GetStockObject(SYSTEM_FIXED_FONT), 0L );

  hdc = GetDC( hwind );
  SelectObject( hdc, GetStockObject( SYSTEM_FIXED_FONT ) );
  GetTextMetrics( hdc, &tm );
  texth = tm.tmHeight + tm.tmExternalLeading+1;
  textw = tm.tmAveCharWidth;
  ReleaseDC( hwind, hdc );

  GetWindowRect( hwind, (LPRECT) &rect );
  MoveWindow( hwind, 1, 1, 72 * textw , lines * texth, TRUE );
  }
